//
//  WDWebObjectDelegate.swift
//  
//
//  Created by Danny on 2022/2/4.
//

import WebKit


/**
 The delegate protocol to receive the result evaluate by `WKWebView`.
 
 Here is a sample code of how to adopt this delegate in your project.
 
 ```swift
 func webView(_ webView: WKWebView, didFinishEvaluateJavaScript result: String) {
     // decode JSON data...
     do {
         let data: [String] = try webObject.decode(jsonString: result)
         // ...
     } catch {
         print(error)
     }
 }
 
 func webView(_ webView: WKWebView, didFailEvaluateJavaScript error: Error) {
     print(error)
 }
 ```
 */
public protocol WDWebObjectDelegate {
    /// Gets called when `func webView(_ webView: WKWebView, didFinish navigation: WKNavigation!)` is called.
    /// - Parameter evaluateJavaScript: the string value return value from JavaScript
    func webView(_ webView: WKWebView, didFinishEvaluateJavaScript result: String)
    
    /// Gets called when `webView.evaluateJavaScript(script)` passed an error.
    /// - Parameter javaScriptError: the error message generated by webView.
    func webView(_ webView: WKWebView, didFailEvaluateJavaScript error: Error)
}
